@@ -2,7 +2,7 @@ import json
 import os
 import shutil
 import subprocess
-from loguru import logger
+
 import hvac
 import requests
 import tldextract
@@ -17,6 +17,27 @@ def rmdir(dir_location):
         logger.info(f"Removed existing config dir {dir_location}")
 
 
+def extract_domain_parts(fqdn):
+    fqdn_parts = tldextract.extract(fqdn)
+    return fqdn_parts
+
+
+def write_into_file(file_content, file_location):
+    with open(file_location, "w") as f:
+        f.write(file_content)
+
+
+def prepare_terraform_environment(data_dir):
+    os.environ["PATH"] += os.pathsep + f"{os.path.expanduser('~')}/bin"
+    os.environ["TF_REGISTRY_DISCOVERY_RETRY"] = "10"
+    os.environ["TF_REGISTRY_CLIENT_TIMEOUT"] = "60"
+    os.environ["TF_IN_AUTOMATION"] = "true"
+    os.environ["TF_PLUGIN_CACHE_DIR"] = f"{os.path.expanduser('~')}/.terraform.d/plugin-cache"
+    os.environ["TF_DATA_DIR"] = data_dir
+    if not os.path.exists(f"{os.path.expanduser('~')}/.terraform.d/plugin-cache"):
+        os.makedirs(f"{os.path.expanduser('~')}/.terraform.d/plugin-cache")
+
+
 class RundeckJobs(object):
     def __init__(
             self,
@@ -24,10 +45,13 @@ class RundeckJobs(object):
     ):
         self.uptimerobot_api_url = uptimerobot_api_url
 
-    wait_random_range: tuple = (60, 180)
-    number_of_retries: int = 3
+    min_retry_delay = int(os.environ.get("RD_OPTION_MIN_RETRY_DELAY", 60))
+    max_retry_delay = int(os.environ.get("RD_OPTION_MIN_RETRY_DELAY", 180))
+    number_of_retries = int(os.environ.get("RD_OPTION_NUMBER_OF_RETRIES", 3))
 
-    @retry(wait=wait_random(*wait_random_range), stop=stop_after_attempt(3))
+    wait_random_range: tuple = (min_retry_delay, max_retry_delay)
+
+    @retry(wait=wait_random(*wait_random_range), stop=stop_after_attempt(number_of_retries))
     def run_command(self, command, cwd, check=True):
         subprocess.run(command, cwd=cwd, check=check)
 
@@ -99,8 +123,8 @@ class RundeckJobs(object):
 
         backend_config_file_location = f"/tmp/{fqdn}.backend.tfvars"
         backend_config_file_content = f'''conn_str = "{pg_backend_conn_str}"'''
-        with open(backend_config_file_location, "w") as f:
-            f.write(backend_config_file_content)
+        write_into_file(backend_config_file_content,
+                        backend_config_file_location)
 
         vars_file_location = f"/tmp/{fqdn}.tfvars"
         vars_file_content = f"""
@@ -115,8 +139,7 @@ class RundeckJobs(object):
         vcs_repo_oauth_token_id = "{vcs_repo_oauth_token_id}"
         tag_names = {json.dumps(tag_names.split(','))}
         """
-        with open(vars_file_location, "w") as f:
-            f.write(vars_file_content)
+        write_into_file(vars_file_content, vars_file_location)
 
         os.environ["PATH"] += os.pathsep + f"{os.path.expanduser('~')}/bin"
         os.environ["TF_REGISTRY_DISCOVERY_RETRY"] = "10"
@@ -202,7 +225,7 @@ class RundeckJobs(object):
 
     ):
 
-        fqdn_parts = tldextract.extract(fqdn)
+        fqdn_parts = extract_domain_parts(fqdn)
         sld = f"{fqdn_parts.domain}.{fqdn_parts.suffix}"
         webserver_netdata_fqdn = f"netdata-webserver.{sld}" if fqdn_parts.subdomain == "" else f"netdata-webserver-{fqdn}"
         mariadb_server_netdata_fqdn = f"netdata-dbserver.{sld}" if fqdn_parts.subdomain == "" else f"netdata-dbserver-{fqdn}"
@@ -230,25 +253,26 @@ class RundeckJobs(object):
               name = "%s-%s-%s"
             }
             ''' % (organization, fqdn.replace('.', '_'), deployment_environment, datacenter)
-        with open(backend_config_file_location, "w") as f:
-            f.write(backend_config_file_content)
+        write_into_file(backend_config_file_content,
+                        backend_config_file_location)
 
         os.environ["PATH"] += os.pathsep + f"{os.path.expanduser('~')}/bin"
 
         terraformrc_file_content = '''credentials "app.terraform.io" {
               token = "%s"
             }''' % terraform_cloud_token
-        with open(f"{os.path.expanduser('~')}/.terraformrc", "w") as f:
-            f.write(terraformrc_file_content)
+        write_into_file(terraformrc_file_content,
+                        f"{os.path.expanduser('~')}/.terraformrc")
 
         os.environ["TF_REGISTRY_DISCOVERY_RETRY"] = "10"
         os.environ["TF_REGISTRY_CLIENT_TIMEOUT"] = "60"
         os.environ["TF_IN_AUTOMATION"] = "true"
 
         shutil.copytree(f"{terraform_code_dir}/", f"{config_dir_location}/")
-        self.run_command(["terraform", "init", "-input=false", "-reconfigure", "-force-copy", "-backend-config",
-                          backend_config_file_location], cwd=config_dir_location,
-                         check=True)
+        self.run_command(
+            ["terraform", "init", "-input=false", "-reconfigure", "-force-copy", "-backend-config",
+             backend_config_file_location], cwd=config_dir_location,
+            check=True)
 
         vars_file_location = f"{config_dir_location}/{fqdn}.auto.tfvars"
         vars_file_content = f"""fqdn = "{fqdn}"
@@ -278,8 +302,7 @@ class RundeckJobs(object):
             mariadb_server_type = "{mariadb_server_type}"
             jira_issue_key = "{jira_issue_key}"
             """
-        with open(vars_file_location, "w") as f:
-            f.write(vars_file_content)
+        write_into_file(vars_file_content, vars_file_location)
 
         if deployment_action == "apply":
             self.run_command(["terraform", "apply", "-input=false", "-auto-approve"], cwd=config_dir_location,
@@ -341,7 +364,6 @@ class RundeckJobs(object):
                                            execution_mode=execution_mode, deployment_environment=deployment_environment,
                                            deployment_action=deployment_action)
 
-    @logger.catch
     def nextjs_build(
             self,
             pg_user: str = os.environ.get(
@@ -356,6 +378,8 @@ class RundeckJobs(object):
             organization: str = os.environ.get('RD_OPTION_ORGANIZATION'),
             deployment_action: str = os.environ.get(
                 'RD_OPTION_DEPLOYMENT_ACTION'),
+            deployment_environment: str = os.environ.get(
+                'RD_OPTION_DEPLOYMENT_ENVIRONMENT'),
             git_domain: str = os.environ.get('RD_OPTION_GIT_DOMAIN'),
             git_reponame: str = os.environ.get('RD_OPTION_GIT_REPONAME'),
             git_branch: str = os.environ.get('RD_OPTION_GIT_BRANCH'),
@@ -369,17 +393,19 @@ class RundeckJobs(object):
     ):
         pg_backend_conn_str = f"postgres://{pg_user}:{pg_password}@{pg_ip}:{pg_port}/{pg_db}"
 
-        config_dir_location = f'/tmp/{fqdn}.build'
+        config_dir_location = f'/tmp/{fqdn}.{deployment_environment}.build'
         rmdir(config_dir_location)
+        if os.path.exists(f"{terraform_code_dir}/.terraform.lock.hcl"):
+            os.remove(f"{terraform_code_dir}/.terraform.lock.hcl")
 
-        backend_config_file_location = f"/tmp/{fqdn}.build.backend.tfvars"
+        backend_config_file_location = f"/tmp/{fqdn}.{deployment_environment}.build.backend.tfvars"
         backend_config_file_content = f'''conn_str = "{pg_backend_conn_str}"'''
-        with open(backend_config_file_location, "w") as f:
-            f.write(backend_config_file_content)
+        write_into_file(backend_config_file_content,
+                        backend_config_file_location)
 
-        fqdn_parts = tldextract.extract(fqdn)
+        fqdn_parts = extract_domain_parts(fqdn)
         sld = f"{fqdn_parts.domain}.{fqdn_parts.suffix}"
-        vars_file_location = f"/tmp/{fqdn}.build.tfvars"
+        vars_file_location = f"/tmp/{fqdn}.{deployment_environment}.build.tfvars"
         vars_file_content = f"""
         fqdn = "{fqdn}"
         sld = "{sld}"
@@ -392,19 +418,9 @@ class RundeckJobs(object):
         vault_login_username = "{vault_login_username}"
         vault_login_password = "{vault_login_password}"
         """
-        with open(vars_file_location, "w") as f:
-            f.write(vars_file_content)
+        write_into_file(vars_file_content, vars_file_location)
 
-        os.environ["PATH"] += os.pathsep + f"{os.path.expanduser('~')}/bin"
-        os.environ["TF_REGISTRY_DISCOVERY_RETRY"] = "10"
-        os.environ["TF_REGISTRY_CLIENT_TIMEOUT"] = "60"
-        os.environ["TF_IN_AUTOMATION"] = "true"
-        os.environ["TF_PLUGIN_CACHE_DIR"] = "~/.terraform.d/plugin-cache"
-        os.environ["TF_DATA_DIR"] = config_dir_location
-
-        # shutil.copytree(f"{terraform_code_dir}/", f"{config_dir_location}/")
-        if not os.path.exists("~/.terraform.d/plugin-cache"):
-            os.makedirs("~/.terraform.d/plugin-cache")
+        prepare_terraform_environment(config_dir_location)
         self.run_command(["terraform", "init", "-input=false", "-reconfigure", "-force-copy",
                           f"-backend-config", backend_config_file_location],
                          cwd=terraform_code_dir, check=True)
@@ -433,3 +449,101 @@ class RundeckJobs(object):
                               "default"], cwd=terraform_code_dir, check=True)
             self.run_command(["terraform", "workspace", "delete", f"{fqdn}.build"],
                              cwd=terraform_code_dir, check=True)
+
+    @retry(wait=wait_random(*wait_random_range), stop=stop_after_attempt(number_of_retries))
+    def nextjs_deploy(
+            self,
+            fqdn: str = os.environ.get("RD_OPTION_FQDN"),
+            git_branch: str = os.environ.get("RD_OPTION_GIT_BRANCH"),
+            datacenter: str = os.environ.get('RD_OPTION_DATACENTER'),
+            organization: str = os.environ.get('RD_OPTION_ORGANIZATION'),
+            execution_mode: str = os.environ.get('RD_OPTION_EXECUTION_MODE'),
+            deployment_environment: str = os.environ.get(
+                'RD_OPTION_DEPLOYMENT_ENVIRONMENT'),
+            deployment_action: str = os.environ.get(
+                "RD_OPTION_DEPLOYMENT_ACTION"),
+            vault_login_username: str = os.environ.get(
+                "RD_OPTION_VAULT_LOGIN_USERNAME"),
+            vault_login_password: str = os.environ.get(
+                "RD_OPTION_VAULT_LOGIN_PASSWORD"),
+            jira_issue_key: str = os.environ.get("RD_OPTION_JIRA_ISSUE_KEY"),
+            terraform_cloud_token: str = os.environ.get(
+                "RD_OPTION_TERRAFORM_CLOUD_TOKEN"),
+            terraform_code_dir: str = f"{os.path.expanduser('~')}/codebase/terraform/rootmodule/nextjs_deploy"
+    ):
+
+        fqdn_parts = extract_domain_parts(fqdn)
+        sld = f"{fqdn_parts.domain}.{fqdn_parts.suffix}"
+        project_name = organization
+        vault_client = hvac.Client(
+            url='https://vault.zadapps.info',
+        )
+        vault_client.login(
+            "/v1/auth/{0}/login/{1}".format("userpass", vault_login_username),
+            json={"password": vault_login_password},
+        )
+
+        config_dir_location = f'/tmp/{fqdn}'
+        rmdir(config_dir_location)
+        if os.path.exists(f"{terraform_code_dir}/.terraform.lock.hcl"):
+            os.remove(f"{terraform_code_dir}/.terraform.lock.hcl")
+
+        backend_config_file_location = f"/tmp/{fqdn}.backend.tfvars"
+        backend_config_file_content = '''
+            organization = "%s"
+            workspaces {
+              name = "%s-%s-%s"
+            }
+            ''' % (organization, fqdn.replace('.', '_'), deployment_environment, datacenter)
+        write_into_file(backend_config_file_content,
+                        backend_config_file_location)
+
+        os.environ["PATH"] += os.pathsep + f"{os.path.expanduser('~')}/bin"
+
+        terraformrc_file_content = '''credentials "app.terraform.io" {
+              token = "%s"
+            }''' % terraform_cloud_token
+        write_into_file(terraformrc_file_content,
+                        f"{os.path.expanduser('~')}/.terraformrc")
+
+        prepare_terraform_environment(config_dir_location)
+        self.run_command(
+            ["terraform", "init", "-input=false", "-reconfigure", "-force-copy", "-backend-config",
+             backend_config_file_location], cwd=terraform_code_dir,
+            check=True)
+
+        vars_file_location = f"{config_dir_location}/{fqdn}.auto.tfvars"
+        vars_file_content = f"""fqdn = "{fqdn}"
+            git_branch = "{git_branch}"
+            project_name = "{project_name}"
+            sld = "{sld}"
+            deployment_environment = "{deployment_environment}"
+            vault_login_username = "{vault_login_username}"
+            vault_login_password = "{vault_login_password}"
+            jira_issue_key = "{jira_issue_key}"
+            """
+        write_into_file(vars_file_content, vars_file_location)
+
+        if deployment_action == "apply":
+            self.run_command(["terraform", "apply", "-input=false", "-auto-approve", "-var-file", vars_file_location],
+                             cwd=terraform_code_dir,
+                             check=True)
+        elif deployment_action == "destroy":
+            self.run_command(["terraform", "destroy", "-input=false", "-auto-approve", "-var-file", vars_file_location],
+                             cwd=terraform_code_dir,
+                             check=True)
+            pg_creds = vault_client.secrets.kv.read_secret_version(
+                path=f'tenant/devops/{project_name}/terraform-pgsql')['data']['data']
+            self.terraform_cloud_workspace(pg_user=pg_creds['user'],
+                                           pg_password=pg_creds['password'],
+                                           pg_ip=pg_creds['ip'],
+                                           pg_port=pg_creds['port'],
+                                           pg_db=pg_creds['db_name'],
+                                           fqdn=fqdn, organization=organization,
+                                           datacenter=datacenter,
+                                           tag_names=os.environ.get(
+                                               'RD_OPTION_TAG_NAMES'),
+                                           with_vcs_repo=os.environ.get(
+                                               'RD_OPTION_WITH_VCS_REPO'),
+                                           execution_mode=execution_mode, deployment_environment=deployment_environment,
+                                           deployment_action=deployment_action)
